/*
File      : hwio.cpp
Project   : XCTL
Author    : Jan Picard <picard@informatik.hu-berlin.de> 2001-2002
   Institut fuer Informatik,
   Humboldt-Universitaet Berlin
Content   : another level of indirection:
         providing port I/O depending on Windows version
         - for win16 direct inp/outp calls are allowed
         - for win32 we use xport, a free generic kernel mode driver
           downloadable from www.simtel.net, search for "xport"
*/

#include "utils\u_utils.h"
#include "porting\types.h"
#include "hardware\hwguids.h"
#include "hardware\hwio.h"
#include "motrstrg\motorcontroller.h"
#include "detecuse\detectorcontroller.h"

#include "utils\CheckMemory.h"	// CHECKMEM

HINSTANCE hModuleInstance;

char _str_[512];
// dll version numbering scheme is still not decided
static char hwVersion[ ]= "V 1.00 ("__DATE__")";

BOOL WINAPI DllMain(HINSTANCE hModule, DWORD dwFunction, LPVOID)
{
	switch (dwFunction)
	{
		case DLL_PROCESS_ATTACH:
			// Initialize function array.
			hModuleInstance= hModule;
			break;

		case DLL_PROCESS_DETACH:
			//CHECKMEM("HWIO.DLL");	//hp
			break;
	};
	return TRUE;
}

//############################################################################
// Code for dll handling
//############################################################################

LPCSTR _HWIOCLASS WINAPI hwGetVersion( void )
{
	return ( LPCSTR ) hwVersion;
};

HINSTANCE _HWIOCLASS WINAPI hwGetInstance( void )
{
	return hModuleInstance;
};

/////////////////////////////////////////////////////////////////
_HWIOCLASS void PrintError(DWORD dwError)
{
	LPVOID lpMsgBuf;

	if (!dwError)
		dwError= GetLastError();

	//Systemfehler und entsprechende Meldung ermitteln
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
				  0, dwError, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &lpMsgBuf, 0, 0);

	//Fehlermeldung ausgeben
	MessageBox(0, (LPCTSTR)lpMsgBuf, "XControl", MBFAILURE);

	//Puffer freigeben
	if(lpMsgBuf)
		HeapFree(GetProcessHeap(), 0, lpMsgBuf);
	lpMsgBuf= 0;
}

/////////////////////////////////////////////////////////////////
_HWIOCLASS DWORD EnumDevices(const GUID& guid)
{
	LPGUID pGUID= const_cast<LPGUID>(&guid);
	DWORD devIndex= 0;
	HDEVINFO info= SetupDiGetClassDevs(pGUID, 0, 0, DIGCF_PRESENT | DIGCF_INTERFACEDEVICE);

	if (info == INVALID_HANDLE_VALUE)
	{
		PrintError();
		return ( -1);
	}

	SP_INTERFACE_DEVICE_DATA ifData;
	ifData.cbSize= sizeof(ifData);

	while (SetupDiEnumDeviceInterfaces(info, 0, pGUID, devIndex, &ifData))
		devIndex++;

	DWORD dwError= GetLastError();
	if (dwError != ERROR_NO_MORE_ITEMS)
	{
		PrintError(dwError);
		SetupDiDestroyDeviceInfoList(info);
		return ( -1);
	}

	SetupDiDestroyDeviceInfoList(info);
	return (devIndex);
}

/////////////////////////////////////////////////////////////////
_HWIOCLASS bool GetDeviceDriver(const GUID& guid, DWORD devIndex, TCHAR** pDriverName)
{
	LPGUID pGUID= const_cast<LPGUID>(&guid);
	HDEVINFO info= SetupDiGetClassDevs(pGUID, 0, 0, DIGCF_PRESENT | DIGCF_INTERFACEDEVICE);

	if (info == INVALID_HANDLE_VALUE)
	{
		PrintError();
		return (false);
	}

	SP_INTERFACE_DEVICE_DATA ifData;
	ifData.cbSize= sizeof(ifData);

	if (!SetupDiEnumDeviceInterfaces(info, 0, pGUID, devIndex, &ifData))
	{
		PrintError();
		SetupDiDestroyDeviceInfoList(info);
		return (false);
	}

	DWORD dwSize= 0;
	SetupDiGetDeviceInterfaceDetail(info, &ifData, 0, 0, &dwSize, 0);

	PSP_INTERFACE_DEVICE_DETAIL_DATA devDetail= (PSP_INTERFACE_DEVICE_DETAIL_DATA)HeapAlloc(GetProcessHeap(), 0, dwSize);
	if (!devDetail)
	{
		MessageBox(0, "Zu wenig Hauptspeicher", "XControl::GetDeviceDriver", MBFAILURE);
		SetupDiDestroyDeviceInfoList(info);
		return (false);
	}
	devDetail->cbSize= sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

	SP_DEVINFO_DATA devData;
	devData.cbSize= sizeof(SP_DEVINFO_DATA);

	if (!SetupDiGetDeviceInterfaceDetail(info, &ifData, devDetail, dwSize, 0, &devData))
	{
		PrintError();
		if(devDetail)
			HeapFree(GetProcessHeap(), 0, devDetail);
		devDetail= 0;
		SetupDiDestroyDeviceInfoList(info);
		return (false);
	}

	dwSize= _tcsclen(devDetail->DevicePath) + 1;
	*pDriverName= (TCHAR*)HeapAlloc(GetProcessHeap(), 0, dwSize);
	if (!pDriverName)
	{
		MessageBox(0, "Zu wenig Hauptspeicher", "XControl::GetDEviceDriver", MBFAILURE);
		if(devDetail)
			HeapFree(GetProcessHeap(), 0, devDetail);
		devDetail= 0;
		SetupDiDestroyDeviceInfoList(info);
		return (false);
	}
	ZeroMemory(*pDriverName, dwSize);
	//_tcsnccpy(*pDriverName,devDetail->DevicePath,dwSize);
	_tcscpy(*pDriverName, devDetail->DevicePath);

	if(devDetail)
		HeapFree(GetProcessHeap(), 0, devDetail);
	devDetail= 0;
	SetupDiDestroyDeviceInfoList(info);

	return (true);
}

/////////////////////////////////////////////////////////////////
bool RegisterDrivers(DeviceList* Drivers, const GUID& guid, EDeviceType DeviceID)
{
	HANDLE hFile= INVALID_HANDLE_VALUE;
	DWORD ReturnedLength= 0;

	DWORD devices= EnumDevices(guid);
	for (DWORD i= 0;i < devices;i++)
	{
		TCHAR* DriverName= 0;
		GetDeviceDriver(guid, i, &DriverName);
		if (DriverName)
		{
			hFile= CreateFile(DriverName, GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
			if (hFile == INVALID_HANDLE_VALUE)
			{
				MessageBox(0, "Treiber konnte nicht geffnet werden.", "XCONTROL", MBFAILURE);
				return (false);
			}

			DWORD ioCode= IOCTL_MC_REPORT_ID;
			DWORD dwHardwareID;

			// Handle to device, IO Control code, Buffer to driver, Length of buffer in bytes,
			// Buffer from driver, Length of buffer in bytes, Bytes placed in DataBuffer,
			// NULL means wait till op. completes.
			if (!DeviceIoControl(hFile, ioCode, 0, 0, &dwHardwareID, sizeof(DWORD), &ReturnedLength, NULL))
			{
				MessageBox(0, "IOCTL_MC_REPORT_ID fehlgeschlagen.", "XCONTROL", MBFAILURE);
				return (false);
			}

			if (!CloseHandle(hFile))
			{
				MessageBox(0, "CloseHandle() fehlgeschlagen.", "XCONTROL", MBFAILURE);
			}

			char szHardwareID[9];
			memset(szHardwareID, '\0', 9);
			itoa(dwHardwareID, szHardwareID, 16);

			Drivers->Add(DeviceID, szHardwareID, DriverName);
		}
	}

	return (true);
}

/////////////////////////////////////////////////////////////////
DeviceListEntry::DeviceListEntry(EDeviceType _DeviceID, LPTSTR _HardwareID, LPTSTR _FileName)
		: prev(0), next(0), DeviceID(_DeviceID)
{
	memset(HardwareID, '\0', 9);
	memset(FileName, '\0', 257);

	if (strnicmp("_HardwareID", "0x", 2) == 0)	//ggf. fhrendes "0x" entfernen
		_HardwareID += 2;
	strncpy(HardwareID, _HardwareID, 8);	//00000000 - FFFFFFFF

	strncpy(FileName, _FileName, 256);
}

void DeviceListEntry::SetPrevEntry(DeviceListEntry* pEntry)
{
	prev= pEntry;
}

void DeviceListEntry::SetNextEntry(DeviceListEntry* pEntry)
{
	next= pEntry;
}

DeviceListEntry* DeviceListEntry::GetPrevEntry()
{
	return (prev);
}

DeviceListEntry* DeviceListEntry::GetNextEntry()
{
	return (next);
}

int DeviceListEntry::GetDeviceID()
{
	return (DeviceID);
}

LPTSTR DeviceListEntry::GetHardwareID()
{
	return (HardwareID);
}

LPTSTR DeviceListEntry::GetFileName()
{
	return (FileName);
}

/////////////////////////////////////////////////////////////////
DeviceList::DeviceList() : first(0), last(0)
{}

DeviceList::~DeviceList()
{
	Clear();
}

void DeviceList::Add(EDeviceType DeviceID, LPTSTR HardwareID, LPTSTR FileName)
{
	DeviceListEntry* pEntry= new DeviceListEntry(DeviceID, HardwareID, FileName);
	if (!first)
	{
		first= pEntry;
		last= pEntry;
	}
	else
	{
		pEntry->SetPrevEntry(last);
		last->SetNextEntry(pEntry);
		last= pEntry;
	}
}

void DeviceList::Add(DeviceListEntry* pEntry)
{
	if (!first)
	{
		first= pEntry;
		last= pEntry;
	}
	else
	{
		pEntry->SetPrevEntry(last);
		last->SetNextEntry(pEntry);
		last= pEntry;
	}
}

void DeviceList::Clear()
{
	DeviceListEntry* temp= first;
	DeviceListEntry* next= 0;
	if (temp)
		next= temp->GetNextEntry();

	while (temp)
	{
		_FREEOBJ(temp);
		temp= next;
		if (temp)
			next= temp->GetNextEntry();
	}
}

LPTSTR DeviceList::GetFileName(int DeviceID, LPTSTR HardwareID)
{
	DeviceListEntry* temp= first;

	if (!temp)
		return (0);

	do
	{
		//DeviceID vergleichen
		if (DeviceID == temp->GetDeviceID())
		{
			//HardwareID vergleichen
			if (strnicmp(HardwareID, "0x", 2) == 0)
			{
				HardwareID= HardwareID + 2;	//Vergleich ohne "0x"
			}

			if (stricmp(HardwareID, temp->GetHardwareID()) == 0)
				return (temp->GetFileName());
		}

		temp= temp->GetNextEntry();
	} while (temp);

	return (0);
}

/////////////////////////////////////////////////////////////////
HardwareIo::HardwareIo() : hDevice(0)
{
}

HardwareIo::HardwareIo(char* _szDeviceName) : hDevice(0)
{
	memset(szDeviceName, '\0', 257);
	if (_szDeviceName)
	{
		strncpy(szDeviceName, _szDeviceName, 256);

		hDevice= CreateFile(szDeviceName, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_DELETE, 0, OPEN_EXISTING, 0, 0);

		if (!hDevice || hDevice == INVALID_HANDLE_VALUE)        // Was the device opened?
		{
			PrintError();
			//throw;
		}
	}
	else
	{
		strncpy(szDeviceName, "", 1);
	}
}

HardwareIo::~HardwareIo()
{
	if (hDevice)
	{
		if (CloseHandle(hDevice) == FALSE)
			PrintError();
		hDevice= 0;
	}
}

BOOL HardwareIo::Read(DWORD Offset, BYTE* Data)
{
	DWORD ReturnedLength;     // Number of bytes returned

	if(!hDevice)
		return(FALSE);

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,              // Handle to device
						   IOCTL_MC_READ_BYTE,  	// Control code for Read
						   &Offset,  			// Buffer to driver.
						   sizeof(Offset),  		// Length of buffer in bytes.
						   Data,  				// Buffer from driver.
						   sizeof(BYTE),         // Length of buffer in bytes.
						   &ReturnedLength,      // Bytes placed in DataBuffer.
						   NULL                // NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	return (IoctlResult);
}

BOOL HardwareIo::Read(DWORD Offset, WORD* Data)
{
	DWORD ReturnedLength;     // Number of bytes returned

	if(!hDevice)
		return(FALSE);

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,              // Handle to device
						   IOCTL_MC_READ_WORD,  	// IO Control code for Read
						   &Offset,  			// Buffer to driver.
						   sizeof(Offset),  		// Length of buffer in bytes.
						   Data,  		        // Buffer from driver.
						   sizeof(WORD),         // Length of buffer in bytes.
						   &ReturnedLength,      // Bytes placed in DataBuffer.
						   NULL                // NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	return (IoctlResult);
}

BOOL HardwareIo::Read(DWORD Offset, DWORD* Data)
{
	DWORD ReturnedLength;     // Number of bytes returned

	if(!hDevice)
		return(FALSE);

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,              // Handle to device
						   IOCTL_MC_READ_DWORD,  // IO Control code for Read
						   &Offset,  			// Buffer to driver.
						   sizeof(Offset),  		// Length of buffer in bytes.
						   Data,  		        // Buffer from driver.
						   sizeof(DWORD),        // Length of buffer in bytes.
						   &ReturnedLength,      // Bytes placed in DataBuffer.
						   NULL                // NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	return (IoctlResult);
}

BOOL HardwareIo::Write(DWORD Offset, BYTE Data)
{
	DWORD ReturnedLength;     // Number of bytes returned

	if(!hDevice)
		return(FALSE);

	WriteBuffer DataBuf;
	DataBuf.Offset= Offset;
	DataBuf.bData= Data;

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,              // Handle to device
						   IOCTL_MC_WRITE_BYTE,  // IO Control code for Read
						   &DataBuf,  			// Buffer to driver.
						   sizeof(DataBuf),  	// Length of buffer in bytes.
						   0,  			        // Buffer from driver.
						   0,  					// Length of buffer in bytes.
						   &ReturnedLength,      // Bytes placed in DataBuffer.
						   NULL                // NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	return (IoctlResult);
}

BOOL HardwareIo::Write(DWORD Offset, WORD Data)
{
	DWORD ReturnedLength;     // Number of bytes returned

	if(!hDevice)
		return(FALSE);

	WriteBuffer DataBuf;
	DataBuf.Offset= Offset;
	DataBuf.wData= Data;

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,              // Handle to device
						   IOCTL_MC_WRITE_WORD,  // IO Control code for Read
						   &DataBuf,  			// Buffer to driver.
						   sizeof(DataBuf),  	// Length of buffer in bytes.
						   0,  			        // Buffer from driver.
						   0,  					// Length of buffer in bytes.
						   &ReturnedLength,      // Bytes placed in DataBuffer.
						   NULL                // NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	return (IoctlResult);
}

BOOL HardwareIo::Write(DWORD Offset, DWORD Data)
{
	DWORD ReturnedLength;     // Number of bytes returned

	if(!hDevice)
		return(FALSE);

	WriteBuffer DataBuf;
	DataBuf.Offset= Offset;
	DataBuf.dwData= Data;

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,              // Handle to device
						   IOCTL_MC_WRITE_DWORD,  // IO Control code for Read
						   &DataBuf,  			// Buffer to driver.
						   sizeof(DataBuf),  	// Length of buffer in bytes.
						   0,  			        // Buffer from driver.
						   0,  					// Length of buffer in bytes.
						   &ReturnedLength,      // Bytes placed in DataBuffer.
						   NULL                // NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	return (IoctlResult);
}

/////////////////////////////////////////////////////////////////
DummyIo::DummyIo(int DeviceID, LPTSTR HardwareID, bool DoMessage) : HardwareIo(0)
{
	char DeviceName[257];
	memset(DeviceName, '\0', 257);

	switch (DeviceID)
	{
		case C812:
			strcpy(DeviceName, "C-812");
			break;

		case C832:
			strcpy(DeviceName, "C-832");
			break;

		case RADICON:
			strcpy(DeviceName, "Radicon");
			break;

		case GENERIC:
			strcpy(DeviceName, "TAm9513");
			break;

		case BRAUN:
			strcpy(DeviceName, "BraunPsd");
			break;

		case STOE:
			strcpy(DeviceName, "StoePsd");
			break;

		default:
			strcpy(DeviceName, "Unbekanntes Gert");
	}

	if (DoMessage)
		MessageBox(0, "Kein Treiber gefunden.", DeviceName, MBFAILURE);
}

/////////////////////////////////////////////////////////////////
BraunPsdIo::BraunPsdIo()
{
}

BraunPsdIo::BraunPsdIo(char* _szDeviceName) : HardwareIo(_szDeviceName)
{}

BraunPsdIo::~BraunPsdIo()
{
	if (hDevice)
	{
		if (CloseHandle(hDevice) == FALSE)
			PrintError();
		hDevice= 0;
	}
}

BOOL BraunPsdIo::Read(DWORD Offset, BYTE* Data)
{
	DWORD ReturnedLength;     // Number of bytes returned

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,              // Handle to device
						   IOCTL_MC_READ_BYTE,  	// Control code for Read
						   &Offset,  			// Buffer to driver.
						   sizeof(Offset),  		// Length of buffer in bytes.
						   Data,  				// Buffer from driver.
						   sizeof(BYTE),         // Length of buffer in bytes.
						   &ReturnedLength,      // Bytes placed in DataBuffer.
						   NULL                // NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	return (IoctlResult);
}

BOOL BraunPsdIo::Write(DWORD Offset, BYTE Data)
{
	DWORD ReturnedLength;     // Number of bytes returned

	WriteBuffer DataBuf;
	DataBuf.Offset= Offset;
	DataBuf.bData= Data;

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,              // Handle to device
						   IOCTL_MC_WRITE_BYTE,  // IO Control code for Read
						   &DataBuf,  			// Buffer to driver.
						   sizeof(DataBuf),  	// Length of buffer in bytes.
						   0,  			        // Buffer from driver.
						   0,  					// Length of buffer in bytes.
						   &ReturnedLength,      // Bytes placed in DataBuffer.
						   NULL                // NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	return (IoctlResult);
}

BOOL BraunPsdIo::SetTimeout(unsigned long Cycles)
{
	DWORD ReturnedLength;     // Number of bytes returned

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,  				// Handle to device
						   IOCTL_DC_SET_WFR_CYCLES,  // IO Control code
						   &Cycles,  				// Buffer to driver.
						   sizeof(Cycles),  			// Length of buffer in bytes.
						   0,  						// Buffer from driver.
						   0,  						// Length of buffer in bytes.
						   &ReturnedLength,  		// Bytes placed in DataBuffer.
						   NULL					// NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	return (IoctlResult);
}

BOOL BraunPsdIo::GetData(IrpParamsGetData *Params) // 31.06.2004 MEMCORRECT (IrpParamsGetData *Params)
{
	DWORD ReturnedLength;     // Number of bytes returned

	DWORD Offset= 0;
	unsigned long Count= 0;

	BOOL IoctlResult= DeviceIoControl(
						   hDevice,  					// Handle to device
						   IOCTL_DC_GET_DATA,  			// IO Control code
						   Params,  						// Buffer to driver.
						   sizeof(IrpParamsGetData),  	// Length of buffer in bytes.
						   &Count,  						// Buffer from driver.
						   sizeof(Count),  				// Length of buffer in bytes.
						   &ReturnedLength,  			// Bytes placed in DataBuffer.
						   NULL						// NULL means wait till op. completes.
					   );

	if (!IoctlResult)
	{
		PrintError();
	}

	if ( !ReturnedLength )
	{
		IoctlResult= FALSE;
		MessageBox(0, "Zeitberschreitung beim Auslesen der Messwerte.", "BraunPsd", MBFAILURE);
	}

	Params->MaxCount= Count;

	return (IoctlResult);
}

/////////////////////////////////////////////////////////////////
BraunPsdDummyIo::BraunPsdDummyIo()
{
	MessageBox(0, "Kein Treiber gefunden.", "BraunPsd", MBFAILURE);
}

/////////////////////////////////////////////////////////////////
BraunPsdDummyIo::BraunPsdDummyIo(LPTSTR HardwareID)
{
	MessageBox(0, "Kein Treiber gefunden.", "BraunPsd", MBFAILURE);
}

/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
Controller::Controller(EDeviceType _DeviceID, LPTSTR _HardwareID, unsigned _MaxClients)
		: Hardware(0), DeviceID(_DeviceID), MaxClients(_MaxClients), Clients(0)
{
	memset(HardwareID, '\0', 9);

	if (strnicmp(_HardwareID, "0x", 2) == 0)	//ggf. fhrendes "0x" entfernen
		_HardwareID += 2;

	strncpy(HardwareID, _HardwareID, 8);
}

int Controller::GetDeviceID()
{
	return (DeviceID);
}

LPTSTR Controller::GetHardwareID()
{
	return (HardwareID);
}

BOOL Controller::AddClient()
{
	if (Clients < MaxClients)
	{
		Clients++;
		return (TRUE);
	}
	else
		return (FALSE);
}

BOOL Controller::Read(DWORD Offset, BYTE* Data)
{
	if (!Hardware)
		return (FALSE);

	return (Hardware->Read(Offset, Data));
}

BOOL Controller::Read(DWORD Offset, WORD* Data)
{
	if (!Hardware)
		return (FALSE);

	return (Hardware->Read(Offset, Data));
}

BOOL Controller::Read(DWORD Offset, DWORD* Data)
{
	if (!Hardware)
		return (FALSE);

	return (Hardware->Read(Offset, Data));
}

BOOL Controller::Write(DWORD Offset, BYTE Data)
{
	if (!Hardware)
		return (FALSE);

	return (Hardware->Write(Offset, Data));
}

BOOL Controller::Write(DWORD Offset, WORD Data)
{
	if (!Hardware)
		return (FALSE);

	return (Hardware->Write(Offset, Data));
}

BOOL Controller::Write(DWORD Offset, DWORD Data)
{
	if (!Hardware)
		return (FALSE);

	return (Hardware->Write(Offset, Data));
}

/////////////////////////////////////////////////////////////////
StandardController::StandardController(EDeviceType _DeviceID, LPTSTR _HardwareID, LPTSTR szDeviceName)
		: Controller(_DeviceID, _HardwareID, 1)
{
	Hardware= new HardwareIo(szDeviceName);
}

StandardController::StandardController(EDeviceType _DeviceID, LPTSTR _HardwareID, DeviceList* Devices)
		: Controller(_DeviceID, _HardwareID, 1)
{
	char* szDeviceName= Devices->GetFileName(DeviceID, HardwareID);
	if (!szDeviceName)
	{
		Hardware= new DummyIo(DeviceID, HardwareID);
	}
	else
	{
		Hardware= new HardwareIo(szDeviceName);
	}

}

StandardController::~StandardController()
{
	_FREEOBJ(Hardware);
}

BOOL StandardController::Check()
{
	return (TRUE);
}

/////////////////////////////////////////////////////////////////
ControllerListEntry::ControllerListEntry(Controller* _asController)
		: prev(0), next(0), asController(_asController)
{}

ControllerListEntry::~ControllerListEntry()
{
	_FREEOBJ(asController);
}

void ControllerListEntry::SetPrevEntry(ControllerListEntry* pEntry)
{
	prev= pEntry;
}

void ControllerListEntry::SetNextEntry(ControllerListEntry* pEntry)
{
	next= pEntry;
}

ControllerListEntry* ControllerListEntry::GetPrevEntry()
{
	return (prev);
}

ControllerListEntry* ControllerListEntry::GetNextEntry()
{
	return (next);
}

int ControllerListEntry::GetDeviceID()
{
	return (asController->GetDeviceID());
}

LPTSTR ControllerListEntry::GetHardwareID()
{
	return (asController->GetHardwareID());
}

Controller* ControllerListEntry::GetController()
{
	return (asController);
}

/////////////////////////////////////////////////////////////////
ControllerList::ControllerList() : first(0), last(0)
{}

ControllerList::~ControllerList()
{
	Clear();
}

void ControllerList::Add(Controller* asController)
{
	ControllerListEntry* pEntry= new ControllerListEntry(asController);
	if (!first)
	{
		first= pEntry;
		last= pEntry;
	}
	else
	{
		pEntry->SetPrevEntry(last);
		last->SetNextEntry(pEntry);
		last= pEntry;
	}
}

void ControllerList::Add(ControllerListEntry* pEntry)
{
	if (!first)
	{
		first= pEntry;
		last= pEntry;
	}
	else
	{
		pEntry->SetPrevEntry(last);
		last->SetNextEntry(pEntry);
		last= pEntry;
	}
}

void ControllerList::Clear()
{
	ControllerListEntry* temp= first;
	ControllerListEntry* next= 0;
	if (temp)
		next= temp->GetNextEntry();

	while (temp)
	{
		_FREEOBJ(temp);
		temp= next;
		if (temp)
			next= temp->GetNextEntry();
	}
}

Controller* ControllerList::GetController(int DeviceID, LPTSTR HardwareID)
{
	ControllerListEntry* temp= first;

	if (!temp)
		return (0);

	if (strnicmp(HardwareID, "0x", 2) == 0)
		HardwareID += 2;

	do
	{
		//DeviceID vergleichen
		if (DeviceID == temp->GetDeviceID())
		{
			//HardwareID vergleichen
			if (stricmp(HardwareID, temp->GetHardwareID()) == 0)
				return (temp->GetController());
		}

		temp= temp->GetNextEntry();
	} while (temp);

	return (0);
}

Controller* ControllerList::GetController(unsigned index)
{ //index ist Null-basiert
	bool bNull= false;
	ControllerListEntry* temp= first;

	if (!temp)
		return (0);

	for (unsigned i= 0;i < index;i++)
	{
		temp= temp->GetNextEntry();
		if (!temp)
		{
			bNull= true;
			break;
		}
	}

	if (bNull)
		return (0);

	return (temp->GetController());
}

DWORD ControllerList::GetControllerIndex(int DeviceID, LPTSTR HardwareID)
{
	DWORD index= 0;
	ControllerListEntry* temp= first;

	if (!temp)
		return (0);

	if (strnicmp(HardwareID, "0x", 2) == 0)
		HardwareID += 2;

	do
	{
		//DeviceID vergleichen
		if (DeviceID == temp->GetDeviceID())
		{
			//HardwareID vergleichen
			if (stricmp(HardwareID, temp->GetHardwareID()) == 0)
				return (index);
		}

		temp= temp->GetNextEntry();
		index++;
	} while (temp);

	return ( -1);
}

/////////////////////////////////////////////////////////////////

