//##############################################################################
//                                                                            //
// MJ_FUNK.H                                                                  //
//                                                                            //
// Subsystem : neue Manuelle Justage / Funktionalitt                         //
// Benutzung durch andere Subsysteme erforderlich: NEIN                       //
//----------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                                //
// Stand  : 23.11.2002                                                        //
// letzter vollstndiger Test: 20.08.2002                                     //
//                                                                            //
//##############################################################################

#ifndef _MJ_FUNK_H
#define _MJ_FUNK_H

#include "motrstrg\motrstrg.h" // Motorsteuerung
#include "detecuse\detecuse.h" // Detektornutzung

//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--//--||--\\--||--

//##############################################################################
// globale Typdeklarationen
//##############################################################################

enum EMotionType { mtDirect= 1, mtDrive= 2, mtStep= 3 }; // 0 zur Kennzeichnung einer unbekannten Betriebsart
enum EDirection { dUp= 1, dDown= 2 };
typedef double TAngle;
typedef double TSpeed;
typedef int TChannel;

//##############################################################################
// TManJustage
//##############################################################################

class TManJustage
{ // Klasse fr die Funktionalitt der Manuellen Justage

	//------------------------------------------------------------------------------
	// TMotorData
	//------------------------------------------------------------------------------

private:

	struct TMotorData
	{ // Informationen fr jeweils einen Antrieb der Manuelle Justage
		
		TMotor *m_lnkMotor; // Antrieb dessen Daten hier verwaltet werden
		EAxisType m_AxisType; // zugeordneter Antriebstyp
		char m_MotorId[50]; // "MOTOR<Index>" Name der INI-Sektion in der HARDWARE.INI
	
		EMotionType m_MotionType; // aktuelle Betriebsart 

		// aktuelle Bewegungsparameter
		TAngle m_AngleDest;  // SollPosition
		TSpeed m_Speed;      // Geschwindigkeit
		TAngle m_AngleWidth; // Schrittweite

		// Info's ber die Bewegung
		BOOL m_KzMoving; // OnTimer(): wenn m_KzMoving!= IsMoving(), dann hat Bewegung gestartet oder gestoppt

		// Speeding (zwischenspeichern der Daten zwischen BeginUpdate und EndUpdate erspart das stndige/ langsame Auslesen beim Antrieb)
		TAngle m_SpAngle;  // letzte Antriebsposition
		BOOL   m_HasAngle; // FALSE wenn die letzte Antriebsposition seit BeginUpdate noch NICHT in m_SpAngle zwischengespeichert wurde
		BOOL m_SpMoving;  // letzter Antriebsstatus (in Bewegung?)
		BOOL m_HasMoving; // FALSE wenn der letzte Antriebsstatus seit BeginUpdate noch NICHT in m_SpMoving zwischengespeichert wurde
	};

	//------------------------------------------------------------------------------
	// Konstruktor/Destruktor
public:

	TManJustage( void );
	virtual  ~TManJustage( void );

	//------------------------------------------------------------------------------
	// Antriebssteuerung
public:

	BOOL DoDirect ( const int ); // Bewegung im Direktbetrieb
	BOOL DoDrive ( const int, const EDirection ); // Bewegung im Fahrbetrieb
	BOOL DoStep ( const int, const EDirection ); // Bewegung im Schrittbetrieb
	BOOL DoStop ( const int ); // stoppen der Bewegung, unabhngig von der Betriebsart
	void DoStopEverything ( void ); // Stoppen aller Antriebe, der Halbwertsbreitenmessung und der Detektormessung

	//------------------------------------------------------------------------------
	// Statusinformationen
public:

	BOOL IsIndexValid ( const int ) const; // gltiger Index der Antriebsliste ?
	BOOL HasOffset ( const int, BOOL& ) const; // Offset gesetzt, d.h. (Offset != 0) ?
	BOOL IsMoving ( const int, BOOL& ); // Antrieb in Bewegung ?
	BOOL IsCalibrated ( const int, BOOL& ) const; // Antrieb kalibriert?

	//------------------------------------------------------------------------------
	// GET-Methoden
public:

	// lesen der aktuellen Betriebsart und der Bewegungsparameter
	EMotionType GetMotionType ( const int, BOOL& ) const; // Betriebsart
	TAngle GetAngleDest ( const int, BOOL& ) const; // Sollposition
	TSpeed GetSpeed ( const int, BOOL& ) const; // Geschwindigkeit
	TAngle GetAngleWidth ( const int, BOOL& ) const; // Schrittweite
	BOOL GetKzMoving ( const int, BOOL& ) const; // OnTimer() will auf Start/ Stop der Bewegung reagiert
	TAngle GetOffset ( const int, BOOL& ) const; // aktuellen Offset fr <Antrieb>
	UINT GetMotorCount ( void ) const; // Anzahl der Antriebe in der Antriebsliste
	LPCSTR GetMotorName ( const int, BOOL& ) const; // Antriebsbezeichnung per standardisierter Antriebsbezeichnung
	int GetMotorIdx ( const LPCSTR ) const; // Index des angegebenen Antriebs; ggf. -1
	int GetMotorIdx ( const EAxisType ) const; // Index des angegebenen Antriebs; ggf. -1

	// lesen der relativen Positionsdaten als Winkelangabe
	TAngle GetAngle ( const int, BOOL& ); // aktuelle Antriebsposition
	TAngle GetAngleMin ( const int, BOOL& ) const; // minimale Antriebsposition
	TAngle GetAngleMax ( const int, BOOL& ) const; // maximale Antriebsposition

	UINT GetDigits ( const int, BOOL& ) const; // lesen der Nachkommastellen
	LPCSTR GetUnit ( const int, BOOL& ) const; // lesen der Einheit

	EAxisType GetHwbAxis(); // Antrieb der zur Halbwertsbreitenmessung verwendet wird
	static UINT ParsingAxis( const LPCSTR ); // lesen der standardisierten Antriebsbezeichnung

	//------------------------------------------------------------------------------
	// SET-Methoden
public:

	// setzen der aktuellen Betriebsart und der Bewegungsparameter
	BOOL SetMotionType ( const int, EMotionType& ); // Betriebsart
	BOOL SetAngleDest ( const int, TAngle& ); // Sollposition
	BOOL SetSpeed ( const int, TSpeed& ); // Geschwindigkeit
	BOOL SetAngleWidth ( const int, TAngle& ); // Schrittweite
	BOOL SetKzMoving ( const int, const BOOL ); // OnTimer() hat auf Start/ Stop der Bewegung reagiert

	//------------------------------------------------------------------------------
	// Offset bzw relative Null
public:

	BOOL SetRelativeZero ( const int ); // setzen der relativen Null
	BOOL ResetRelativeZero ( const int ); // aufheben der relativen Null
	BOOL SetOffset ( const int, TAngle& ); // setzen des Offset fr <Antrieb>

	// Berechnungen zum Offset fr <Antrieb>
	TAngle CalcOffsetFromAngle ( const int, TAngle& ) const; // Berechnung des Offset aus einer Position
	TAngle CalcAngleFromOffset ( const int, TAngle& ) const; // Berechnung der Position aus dem Offset

	//------------------------------------------------------------------------------
	// Kanal-Offset
public:

	int GetDetectorAxisIdx ( void ) const; // Index DES PSD-Antriebs; ggf. -1
	LPCSTR GetDetectorName ( BOOL& ) const; // gibt die Bezeichnung des Detektors zurck; ggf. "PSD"
	BOOL HasDetectorAxis ( void ) const; //  angeschlossen?
	UINT GetDetectorDigits ( BOOL& ) const; // Nachkommastellen der Kanalbreite

	int GetChannel ( void ) const;
	TAngle GetAnglePerChannel ( void ) const; // lesen des aktuellen Verhltnis: Winkel pro Kanal
	BOOL SetChannel ( int& ); // setzen des Kanals

	// Berechnungen zum PSD-Offset
	TAngle CalcChannelOffset ( TChannel& ) const;
	TAngle CalcChannelAngle ( TChannel& ) const;
private:
	BOOL ResetChannelOffset( const int ); // aufheben des Kanaloffsets

	//------------------------------------------------------------------------------
	// Halbwertsbreitenmessung
public:

	BOOL DoStartMeasure ( const HWND ); // starten
	BOOL DoStopMeasure ( void ); // stoppen

	BOOL IsMeasuring ( void ) const; // ==> TRUE <--> Messung aktiv
	BOOL IsMeasureReset ( void ) const; // Messung abgebrochen?
	BOOL GetKzMeasuring ( void ) const; // OnTimer() will auf Start/ Stop der Makroverarbeitung reagiert
	BOOL GetMeasureProgress ( int&, LPSTR, int& ) const; //Index und Anzahl der Kommandos und Statusinformationen bzw. Kommandonamne werden per Parameter zurckgegeben; ==> TRUE <--> Messung aktiv
	double GetMeasureHwb ( void ) const; // ==> HWB oder 0.0
	TDetector *GetDetector ( void ) const; // Zeiger fr den ausgewhlten Detektor, der fr die Halbwertsbreitenmessung bentigt wird
	void SetKzMeasuring ( const BOOL ); // OnTimer() hat auf Start/ Stop der Makroverarbeitung reagiert
	void UpdateDetector ( void ) const; // informiert die Makroverarbeitung, dass neue Messwerte vorliegen

	//------------------------------------------------------------------------------
	// Zustand fr Bewegung ermitteln
public:

	// knnen die folgenden Funktionalitten durchgefhrt werden?
	// das freigeben bzw. sperren und ausgrauen von Steuerelementen
	BOOL CanDoDirect ( const int ); // Direktbetrieb mglich ?
	BOOL CanDoDrive ( const int, const EDirection ); // Fahrbetrieb mglich ?
	BOOL CanDoStep ( const int, const EDirection ); // Schrittbetrieb mglich ?
	BOOL CanDoStop ( const int ); // kann Bewegung gestoppt werden ?

	//------------------------------------------------------------------------------
	// Zustand zum Setzen von Parametern
public:

	BOOL CanSetMotionType ( const int ); // setzen der Betriebsart mglich ?
	BOOL CanSetAngleDest ( const int ); // setzen der Sollposition mglich ?
	BOOL CanSetSpeed ( const int ); // setzen der Geschwindigkeit mglich ?
	BOOL CanSetAngleWidth ( const int ); // setzen der Schrittweite mglich ?
	BOOL CanResetRelativeZero ( const int ); // aufheben der Relativen Null mglich ?
	BOOL CanSetOffset ( const int ); // setzen des Offset und der rel.Null fr <Antrieb> mglich ?

	BOOL CanDoStartMeasure ( void ); // starten der Halbwertsbreitenmessung mglich ?
	BOOL CanDoStopMeasure ( void ) const; // stoppen der Halbwertsbreitenmessung mglich ?

	//------------------------------------------------------------------------------
	// Speeding beginnen bzw. beenden
public:
	UINT  BeginUpdate ( void ); // m_UpdateCount erhhen 
	UINT  EndUpdate   ( void ); //               verringern

	//------------------------------------------------------------------------------
	// Auslesen der wirklichen Antriebsdaten ohne Speeding (Aquivalent zu GetAngle und IsMoving)
private:
	TAngle DetermineAngle  ( const int, BOOL& ) const; // aktuelle Antriebsposition beim Antrieb auslesen
	BOOL   DetermineMoving ( const int, BOOL& ) const; // Antriebsstatus (in Bewegung?) beim Antrieb auslesen

	//------------------------------------------------------------------------------
	// Attribute
private:
	TMotorData *m_MotorList;   // Liste der verfgbaren Antriebe

	TOneDimDetector *m_lnkDetector; // Detektor fr den PSD-Offset
	char m_DetectorId[50]; // // "DEVICE<Index>" Name der INI-Sektion in der HARDWARE.INI
	int m_Channel;  // PSD-Kanal fr den Offset

	BOOL m_IsMeasuring; // Status der HWB-Messung
	UINT m_MotorCount;  // Anzahl der Motoren in m_MotorList
	BOOL m_KzMeasuring; // OnTimer(): wenn m_IsMeasuring!= IsMeasuring(), dann hat Makroverarbeitung gestartet oder gestoppt

	UINT m_UpdateCount; // Speeding
};

#endif //__MJ_FUNK.H

//#############################################################################
// LastLine
//#############################################################################

