//#############################################################################
//                                                                           //
// U_LISTS.H                                                                 //
// Subsystem: Utilities - verkettete Listen                                  //
// Benutung durch andere Subsysteme erforderlich: JA                         //
// Implementation: U_LISTS.CPP                                               //
//---------------------------------------------------------------------------//
// Autoren: Thomas Kullmann, Gnther Reinecker                               //
// Stand:   02.04.2004                                                       //
//                                                                           //
//#############################################################################

#ifndef __U_LISTS_H
#define __U_LISTS_H

#include "u_values.h" // fr _CURVECLASS, _FREEOBJ, Trace

//##############################################################################
// TLnkContainer
//##############################################################################

// Kontainer zum Verketten von void* -Elementen als doppelt verkettete Liste
// Zugriff nur durch dieses Subsystem
class TLnkContainer {
	friend class TLnkList;
	friend class TIndList;

private:
	TLnkContainer(void *const aData, TLnkList &aList);

	// ==> stimmen <aList> und <m_List> berein ?
	bool IsElementOf(const TLnkList *const aList) const;

// ATTRIBUT(E)
private:
	void *const m_Data;

	TLnkList &m_List;

	TLnkContainer *m_Prev;
	TLnkContainer *m_Next;
};

//##############################################################################
// TLnkList
//##############################################################################

// doppelt verkettete Liste
// Zugriff nur ber Container und dieses Subsystem
class _CURVECLASS TLnkList { // entfernen von Elementen bislang nur einmal in der Liste vorhandene Daten und <aData>!=0 getestet
private:
	TLnkList(const TLnkList& aList) : m_FailIfNull(aList.m_FailIfNull), m_FailIfExists(aList.m_FailIfExists), m_AutoDelete(aList.m_AutoDelete), m_CheckOnDel(aList.m_CheckOnDel), m_First(0), m_Last(0) {}; // kein Copy-Konstruktor
	TLnkList &operator= (const TLnkList&) { return *this; }; // kein Zuweisungsoperator

public:
	TLnkList(const bool aFailIfNull= true, const bool aFailIfExists= true, const bool aAutoDelete= false, const bool aCheckOnDel= true);
	virtual  ~TLnkList();

	// gibt das Element aus dem Container zurck
	// (1) wenn <m_AutoDelete>==true darf das Element nicht selbst mit delete freigegeben werden!
	// ==> das im Container verpackte Element 
	void *GetAt(TLnkContainer &aContainer) const;

	// fgt <aData> am Anfang der Liste ein
	// (1) wenn <m_FailIfExists>==true und <aData> bereits in der Liste, dann wird <aData> nicht eingefgt
	// (2) wenn <m_FailIfNull>==true und <aData>==0 ist, wird <aData> nicht eingefgt
	// ==> wurde <aData> eingefgt
	//     bei (1) und (2) ist der Rckgabewert false
	bool Insert(void *const aData);

	// fgt <aData> am Ende der Liste ein
	// (1) wenn <m_FailIfExists>==true und <aData> bereits in der Liste, dann wird <aData> nicht eingefgt
	// (2) wenn <m_FailIfNull>==true und <aData>==0 ist, wird <aData> nicht eingefgt
	// ==> wurde <aData> eingefgt
	//     bei (1) und (2) ist der Rckgabewert false
	bool Append(void *const aData);

	// fgt <aData> VOR <aContainer> ein
	// (0) wenn <aContainer>==0 wird am Anfang der Liste eingefgt - siehe Insert
	// (1) wenn <m_FailIfExists>==true und <aData> bereits in der Liste, dann wird <aData> nicht eingefgt
	// (2) wenn <m_FailIfNull>==true und <aData>==0 ist, wird <aData> nicht eingefgt
	// ==> wurde <aData> eingefgt
	//     bei (1) und (2) ist der Rckgabewert false
	bool InsBefore(TLnkContainer *aContainer, void *const aData);

	// fgt <aData> NACH <aContainer> ein
	// (0) wenn <aContainer>==0 wird am Ende der Liste eingefgt - siehe Append
	// (1) wenn <m_FailIfExists>==true und <aData> bereits in der Liste, dann wird <aData> nicht eingefgt
	// (2) wenn <m_FailIfNull>==true und <aData>==0 ist, wird <aData> nicht eingefgt
	// ==> wurde <aData> eingefgt
	//     bei (1) und (2) ist der Rckgabewert false
	bool AddAfter(TLnkContainer *aContainer, void *const aData);

	// entfernt <aContainer>
	// (1) wenn <m_AutoDelete>==true und das in <aContainer> liegende Non-Null-Element einzigartig in der Liste ist, wird das Element entfernt und mit delete freigegeben
	// ==> das entfernte Element
	//     bei (1) ist der Rckgabewert 0
	void *Remove(TLnkContainer &aContainer);

	// sucht das erste Element, das <aData> verwaltet und entfernt dieses
	void *RemoveData(void *aData);

	// entfernt alle Element aus der Liste
	// bei <m_AutoDelete>==true werden die Non-Null-Elemente mit delete freigegeben
	virtual void Clear();

	// entfernt <aContainer> und Folgeelemente
	// bei <m_AutoDelete>==true werden einzigartige Non-Null-Elemente mit delete freigegeben
	void CutList(TLnkContainer &aContainer);

protected:
	virtual void FreeData(void *aData) const; // berschreiben, wenn in der Liste nur Objekte eines bestimmten Typs enthalten sind (dann <aData> auf diesen Typ casten und freigeben)

private:
	// sucht nach dem nchsten Container, der <aData> beinhaltet - die Suche beginnt bei <aStart>
	// ist <aStart>==0, beginnt die Suche am Anfang
	// ==> der gefundene Container; ggf. 0
	TLnkContainer *Find(void *aData, TLnkContainer *aStart=0);

// ATTRIBUT(E)
protected:
	const bool m_FailIfNull; // Null-Elemente drfen nicht eingefgt werden
	const bool m_FailIfExists; // Elemente drfen nicht mehrfach eingefgt werden
	const bool m_AutoDelete; // Non-Null-Elemente mit delete freigeben, wenn letztes Vorkommen mit Remove bzw. Clear entfernt wird
	const bool m_CheckOnDel; // bei automatischen Freigeben ( <m_AutoDelete>==true ) wird geprft, ob weitere Vorkommen in der Liste verbleiben - ggf. wird das Objekt nicht freigegeben, sondern nur aus der Liste entfernt

	TLnkContainer *m_First;
	TLnkContainer *m_Last;
};

//##############################################################################
// TIndList
//##############################################################################

typedef unsigned long TIndex;

// Liste von void* -Elementen; Zugriff per Index, d.h. [0...GetCount()-1]
// ***In der DEBUG-Version wird zustzlich eine TraceErr-Fehlermeldung ausgegeben!
class _CURVECLASS TIndList : protected TLnkList {
public:
	TIndList(const bool aFailIfNull= true, const bool aFailIfExists= true, const bool aAutoDelete= false, const bool aCheckOnDel= true, const TIndex &aTemp= 32);

	// (0) Bei einem zu groen Index wird das letzte Element zurckgegeben.***
	// (E) Ist die Liste leer, wird 0 zurckgegeben.***
	// <aValid> wird true gesetzt, wenn <aIndex> gltig ist und die Liste nicht leer ist; sonst false.
	// ==> das Element an Position <aIndex>	
	void *GetAt(TIndex aIndex, bool &aValid) const;
	
	// WIEVOR, jedoch ohne aValid - d.h. ein zu groer Index oder eine leere Liste sind fr den Aufrufer nicht nachvollziehbar
	void *GetAt(TIndex aIndex) const;	
	void *operator[] (TIndex aIndex) const;

	// fgt <aData> am Anfang bzw. am Ende der Liste ein
	// (1) und (2) und Rckgabewert gelten von TLnkList::Insert bzw. TLnkList::Append
	bool Insert(void *const aData);
	bool Append(void *const aData);

	// fgt <aData> VOR dem Element an Position <aIndex> ein
	// (E) Bei einem zu groen Index oder einer leeren Liste wird am Ende eingefgt.***
	// (1) und (2) und Rckgabewert gelten von TLnkList::InsBefore
	bool InsBefore(TIndex aIndex, void *const aData);

	// fgt <aData> NACH dem Element an Position <aIndex> ein
	// (E) Bei einem zu groen Index oder einer leeren Liste wird am Ende eingefgt.***
	// (1) und (2) und Rckgabewert gelten von TLnkList::AddAfter
	bool AddAfter(TIndex aIndex, void *const aData);

	// entfernt das Element an Position <aIndex>
	// (0) Bei einem zu groen Index wird das letzte Element entfernt.***
	// (E) Ist die Liste leer, wird nur 0 zurckgegeben.***
	// (1) und Rckgabewert gelten von TLnkList::Remove
	void *Remove(TIndex aIndex);

	// entfernt das erste Vorkommen, das <aData> beinhaltet
	// (E) Ist die Liste leer oder wird <aData> nicht gefunden, wird nur 0 zurckgegeben.
	void *RemoveData(void *aData);

	// entfernt das Element an Position <aIndex> und alle Folgeelemente
	// (E) Bei einem zu groen Index oder einer leeren Liste wird keine Aktion ausgefhrt.***
	void CutList(TIndex aCount);

	// wie TLnkList::Clear
	virtual void Clear();

	// ==> Anzahl der Elemente
	TIndex GetCount() const;

private:
	void DoInvalidAccelerator();

	// Hilfemethoden fr Find
	TLnkContainer *FindForward(TIndex, const TIndex, TLnkContainer *) const; // sucht vorwrts
	TLnkContainer *FindBackward(TIndex, const TIndex, TLnkContainer *) const; // sucht rckwrts	

	// ==> der Container an Position <aIndex>
	// (0) Bei einem zu groen Index wird in der RELEASE-Version das letzte Element zurckgegeben.***
	// (E) Ist die Liste leer, wird 0 zurckgegeben.***
	TLnkContainer *FindIdx(TIndex aIndex, bool aAdoptAccelerator= true) const;

// ATTRIBUT(E)
protected:
	TIndex m_Count;

	// zuletzt benutzer Index und der dazugehriger Container
	TIndex m_AccelIndex;
	TLnkContainer *m_AccelContainer;
};

//##############################################################################
// TIdxContainer
//##############################################################################

const cDefaultSize= 16;

// Container der ein Array von void* -Elementen enthlt
// Zugriff nur durch dieses Subsystem
// ***In der DEBUG-Version wird zustzlich eine TraceErr-Fehlermeldung ausgegeben!
class TIdxContainer {
	friend class TIdxList;

private:
	TIdxContainer(const TIndex &aSize= cDefaultSize);
	virtual ~TIdxContainer(); // die noch enthaltenen Elemente werden nicht automatisch freigegeben, nur die dymanische Liste <m_Data>

	// ==> Gre des Arrays
	TIndex GetSize() const;

	// verschiebt alle Elemente ab incl. <aStartIdx> bis zum Ende um eine Position nach hinten
	// (E) Bei einem zu groen Index wird dieser auf den Anfang der Liste gesetzt, d.h. die gesamte Liste wird geshiftet.***
	// ==> das dabei nach hinten rausgeshiftete (ehemalig letzte) Element
	void *ShiftRight(TIndex aStartIdx);

	// verschiebt alle Elemente an incl. <aStartIdx> bis zum Ende um eine Position nach vorn
	// (E) Bei einem zu groen Index wird dieser auf den Anfang der Liste gesetzt, d.h. die gesamte Liste wird geshiftet.***
	// ==> das dabei nach vorn rausgeshiftete Element (ehemalig an Position <aEndIdx>)
	void *ShiftLeft(TIndex aStartIdx);

// ATTRIBUT(E)
private:
	void **m_Data; // dynamisch erzeugte void* -Liste, Gre ist pro Container unvernderlich

	const TIndex m_Size; // Gre der Liste

	TIdxContainer *m_Prev;
	TIdxContainer *m_Next;	
};

//##############################################################################
// TIdxList
//##############################################################################

// Liste von void* -Elementen; Zugriff per Index, d.h. [0...GetCount()-1]
// ***In der DEBUG-Version wird zustzlich eine TraceErr-Fehlermeldung ausgegeben!
class _CURVECLASS TIdxList { // entfernen von Elementen bislang nur einmal in der Liste vorhandene Daten und <aData>!=0 getestet
private:
	TIdxList(const TIdxList& aList) : m_FailIfNull(aList.m_FailIfNull), m_FailIfExists(aList.m_FailIfExists), m_AutoDelete(aList.m_AutoDelete), m_CheckOnDel(aList.m_CheckOnDel), m_First(0), m_Last(0), m_Count(0), m_Used(0), m_Size(aList,m_Size) {}; // kein Copy-Konstruktor
	TIdxList &operator= (const TIdxList&) { return *this; }; // kein Zuweisungsoperator

public:
	TIdxList(const bool aFailIfNull= true, const bool aFailIfExists= true, const bool aAutoDelete= false, const bool aCheckOnDel= true, const TIndex &aSize= cDefaultSize);
	virtual ~TIdxList();

	// (0) Bei einem zu groen Index wird das letzte Element zurckgegeben.***
	// (E) Ist die Liste leer, wird 0 zurckgegeben.***
	// <aValid> wird true gesetzt, wenn <aIndex> gltig ist und die Liste nicht leer ist; sonst false.
	// ==> das Element an Position <aIndex>	
	void *GetAt(TIndex aIndex, bool &aValid) const;
	
	// WIEVOR, jedoch ohne aValid - d.h. ein zu groer Index oder eine leere Liste sind fr den Aufrufer nicht nachvollziehbar
	void *GetAt(TIndex aIndex) const;	
	void *operator[] (TIndex aIndex) const;

	// fgt <aData> am Anfang bzw. am Ende der Liste ein
	// (1) wenn <m_FailIfExists>==true und <aData> bereits in der Liste, dann wird <aData> nicht eingefgt
	// (2) wenn <m_FailIfNull>==true und <aData>==0 ist, wird <aData> nicht eingefgt
	// ==> wurde <aData> eingefgt
	//     bei (1) und (2) ist der Rckgabewert false
	bool Insert(void *const aData);
	bool Append(void *const aData);

	// fgt <aData> VOR dem Element an Position <aIndex> ein
	// (E) Bei einem zu groen Index oder einer leeren Liste vor dem Ende eingefgt.***
	// (1) wenn <m_FailIfExists>==true und <aData> bereits in der Liste, dann wird <aData> nicht eingefgt
	// (2) wenn <m_FailIfNull>==true und <aData>==0 ist, wird <aData> nicht eingefgt
	// ==> wurde <aData> eingefgt
	//     bei (1) und (2) ist der Rckgabewert false
	bool InsBefore(TIndex aIndex, void *const aData);

	// fgt <aData> NACH dem Element an Position <aIndex> ein
	// (E) Bei einem zu groen Index oder einer leeren Liste wird am Ende eingefgt.***
	// (1) wenn <m_FailIfExists>==true und <aData> bereits in der Liste, dann wird <aData> nicht eingefgt
	// (2) wenn <m_FailIfNull>==true und <aData>==0 ist, wird <aData> nicht eingefgt
	// ==> wurde <aData> eingefgt
	//     bei (1) und (2) ist der Rckgabewert false
	bool AddAfter(TIndex aIndex, void *const aData);

	// entfernt das Element an Position <aIndex>
	// (0) Bei einem zu groen Index wird das letzte Element entfernt.***
	// (E) Ist die Liste leer, wird nur 0 zurckgegeben.***
	// (1) und Rckgabewert gelten von TLnkList::Remove
	void *Remove(TIndex aIndex);

	// entfernt das Element an Position <aCount> und alle Folgeelemente
	// (E) Bei einem zu groen Index oder einer leeren Liste wird keine Aktion ausgefhrt.***
	void CutList(const TIndex &aCount);

	// entfernt alle Elemente
	virtual void Clear();

	// ==> Anzahl der Elemente
	TIndex GetCount() const;

protected:
	virtual void FreeData(void *aData) const; // berschreiben, wenn in der Liste nur Objekte eines bestimmten Typs enthalten sind (dann <aData> auf diesen Typ casten und freigeben)

	// sucht Container und Index fr das Element an Position <aIndex>
	// der Container ist der Rckgabewert, <aIndex> entspricht nach dem Aufruf der Position in diesem Container
	// (E) Bei einem zu groen <aIndex> wird das letzte Element zurckgegeben.
	// (0) Ist die Liste leer, wird 0 zurckgegeben.
	TIdxContainer *FindIdx(TIndex &aIndex) const;

	// sucht den ersten Container der <aData> enthlt
	// der Container ist der Rckgabewert, <aIndex> entspricht nach dem Aufruf der Position in diesem Container
	// (0) Ist die Liste leer oder das Element nicht enthalten, wird 0 zurckgegeben.
	TIdxContainer *Find(void *aData, TIndex &aIndex) const;

// ATTRIBUT(E)
protected:
	const bool m_FailIfNull; // Null-Elemente drfen nicht eingefgt werden
	const bool m_FailIfExists; // Elemente drfen nicht mehrfach eingefgt werden
	const bool m_AutoDelete; // Non-Null-Elemente mit delete freigeben, wenn letztes Vorkommen mit Remove bzw. Clear entfernt wird
	const bool m_CheckOnDel; // bei automatischen Freigeben ( <m_AutoDelete>==true ) wird geprft, ob weitere Vorkommen in der Liste verbleiben - ggf. wird das Objekt nicht freigegeben, sondern nur aus der Liste entfernt

	TIdxContainer *m_First;
	TIdxContainer *m_Last;

	TIndex m_Count; // Anzahl der verwalteten Elemente insgesamt
	TIndex m_Used; // Anzahl der Elemente im letzten Container
	TIndex m_Size; // Gre der Arrays in den Containern
};

typedef TIdxList TBaseList;

//##############################################################################
// TIntList
//##############################################################################

// Liste von int -Elementen; Zugriff per Index, d.h. [0...GetCount()-1]
// ***In der DEBUG-Version wird zustzlich eine TraceErr-Fehlermeldung ausgegeben!
class _CURVECLASS TIntList : protected TBaseList {
public:
	// erzeugt eine Liste mit <aCount> leeren Elementen, deren Wert <aDefaultValue> entspricht
	// <aSize> regelt die Gre der Container, in denen die Werte verpackt werden - siehe TIdxContainer
	TIntList(const TIndex &aCount= 0, const int &aDefaultValue= 0, const TIndex &aSize= cDefaultSize);
	TIntList(const TIntList &aList); // Copy-Konstruktor, die eine neue Liste mit den gleichen Elementen erzeugt
	TIntList &operator= (const TIntList &aList); // Zuweisungsoperator, der eine neue Liste mit den gleichen Elementen erzeugt

	// (E) Bei einem zu groen Index oder wenn die Liste leer ist, wird <m_DefaultValue> zurckgegeben.***
	// <aValid> wird true gesetzt, wenn <aIndex> gltig ist und die Liste nicht leer ist; sonst false.
	// ==> das Element an Position <aIndex>	
	int GetAt(const TIndex &aIndex, bool &aValid) const;
	
	// WIEVOR, jedoch ohne aValid - d.h. es ein zu groer Index oder eine leere Liste sind fr den Aufrufer nicht nachvollziehbar, es wird nur <m_DefaultValue> zurckgegeben
	int GetAt(const TIndex &aIndex) const;	

	// Lesezugriff: 
	//   wie GetAt(...)
	// Schreibzugriff: 
	//   (E) Bei einem zu groen Index oder wenn die Liste leer ist, wird kein Wert gesetzt.***
	//       Dazu wird eine Kopie von <m_DefaultValue> zurckgegeben. Diese kann gendert werden, hat aber keine Wirkung auf andere Elemente der Liste oder <m_DefaultValue> selbst.
	int &operator[] (const TIndex &aIndex);

	// setzt <aValue> als Element an Position <aIndex>
	// (E) Bei einem zu groen Index oder wenn die Liste leer ist, wird kein Wert gesetzt.***
	// ==> wurde der Wert gesetzt
	bool SetAt(const TIndex &aIndex, const int &aValue);

	// fgt <aValue> am Anfang bzw. am Ende der Liste ein
	bool Insert(const int &aValue);
	bool Append(const int &aValue);

	// fgt <aValue> VOR bzw. NACH dem Element an Position <aIndex> ein
	bool InsBefore(const TIndex &aIndex, const int &aValue);
	bool AddAfter(const TIndex &aIndex, const int &aValue);

	// ndert die Gre der Liste auf <aSize>, indem entweder <aDefaultValue> am Ende eingefgt werden ODER die letzten Elemente entfernt werden
	void SetCount(const TIndex &aCount, const int &aDefaultValue);
	
	// wie SetCount(aCount, m_DefaultValue)
	void SetCount(const TIndex &aCount);

	// Element entfernen; Gre ermitteln
	TBaseList::Remove;
	TBaseList::CutList;
	TBaseList::Clear;
	TBaseList::GetCount;

private:
	int m_DefaultValue;
};

//##############################################################################
// TFloatList
//##############################################################################

// Liste von float -Elementen; Zugriff per Index, d.h. [0...GetCount()-1]
// Beschreibung siehe TIntList
class _CURVECLASS TFloatList : protected TBaseList {
public:
	TFloatList(const TIndex &aCount= 0, const float &aDefaultValue= 0.0, const TIndex &aSize= cDefaultSize);
	TFloatList(const TFloatList &aList); 
	TFloatList &operator= (const TFloatList &aList); 

	float GetAt(const TIndex &aIndex, bool &aValid) const;
	float GetAt(const TIndex &aIndex) const;	
	float &operator[] (const TIndex &aIndex);
	bool SetAt(const TIndex &aIndex, const float &aValue);

	bool Insert(const float &aValue);
	bool Append(const float &aValue);

	bool InsBefore(const TIndex &aIndex, const float &aValue);
	bool AddAfter(const TIndex &aIndex, const float &aValue);

	void SetCount(const TIndex &aCount, const float &aDefaultValue);
	void SetCount(const TIndex &aCount);

	// Element entfernen; Gre ermitteln
	TBaseList::Remove;
	TBaseList::CutList;
	TBaseList::Clear;
	TBaseList::GetCount;	

private:
	float m_DefaultValue;
};

//##############################################################################
// TPoint
//##############################################################################

//! Struktur eines Elements der Kurve
class _CURVECLASS TPoint { // aus Speicherspargrnden ist Pt nun ein Array statt TFloatList
public:
	TPoint();
	virtual ~TPoint();
	TPoint(const TPoint &aPoint); 
	TPoint &operator= (const TPoint &aPoint);

	// Direktzugriff auf <m_Pt>
	float GetAt(const TIndex &aIndex, bool &aValid) const;
	float GetAt(const TIndex &aIndex) const;	
	float &operator[] (const TIndex &aIndex);
	bool SetAt(const TIndex &aIndex, const float &aValue);

	BOOL Valid;

private:
	enum { s_Size= 3 }; // Anzahl der Elemente in <m_Pt>
	float *m_Pt;
};

//##############################################################################
// TPointList
//##############################################################################

// Liste von TPoint -Elementen; Zugriff per Index, d.h. [0...GetCount()-1]
// Beschreibung siehe TIntList
class _CURVECLASS TPointList : protected TBaseList {
public: 
	TPointList(const TIndex &aCount= 0, const TIndex &aSize= 128);
	virtual ~TPointList();
	TPointList(const TPointList &aList); 
	TPointList &operator= (const TPointList &aList); 

	TPoint &GetAt(const TIndex &aIndex, bool &aValid) const;
	TPoint &GetAt(const TIndex &aIndex) const;	
	TPoint &operator[] (const TIndex &aIndex) const;

	bool Insert(const TPoint &aValue);
	bool Append(const TPoint &aValue);

	bool InsBefore(const TIndex &aIndex, const TPoint &aValue);
	bool AddAfter(const TIndex &aIndex, const TPoint &aValue);

	void SetCount(const TIndex &aCount, const TPoint &aDefaultValue);
	void SetCount(const TIndex &aCount);

	// Element entfernen; Gre ermitteln
	TBaseList::Remove;
	TBaseList::CutList;
	TBaseList::Clear;
	TBaseList::GetCount;

protected:
	virtual void FreeData(void *aData) const;  

private:
	TPoint m_DefaultValue;
};

#endif // __U_LISTS_H